-------------------------------------------------------------------------
-- [init.lua]
-- panel
-------------------------------------------------------------------------
-- Copyright (C) Textmode DMB 2011
-------------------------------------------------------------------------

local _M = {_NAME = "panel", _TYPE = 'module'}

local util = require "panel.util"
local bbox = util.bbox
-------------------------------------------------------------------------

local panel_mt ={__index = _M}

local p_bgcol = util.color(.4,.4,.4)
local p_fgcol = util.color(.0,.0,.0)
local cnt = 0

function _M:new(t)
	t = t or {}
	assert(type(t) == 'table', "new takes a definition table, or a panel instance! was '"..type(t).."'")
	
	local p = {_TYPE='panel',id=cnt}
	setmetatable(p, panel_mt)
	cnt = cnt+1

	p.x, p.y     = t.x        or 0, t.y or 0
	p.width      = t.width    or 64
	p.height     = t.height   or 32
	p.color_bg   = t.color_bg or p_bgcol
	p.color_fg   = t.color_fg or p_fgcol
	p.name       = t.name     or ("Panel#"..cnt)
	p.align      = t.align    or 'center'
	p.valign     = t.valign   or 'middle'
	p.font       = t.font     or 12
	p.scale      = t.scale    or 1
	p.background = t.background
	p.text       = t.text
--	p.visible    = ((t.visible ~= nil) and (not not t.visible)) or true
	p.visible    = true
	if t.border == nil then p.border = true else p.border=t.border end

	p.children = {}

	assert(type(p.x) == 'number' and type(p.y) == 'number', "'x' and 'y' must be numbers!")
	assert(type(p.width) == 'number' and type(p.height) == 'number', "'width' and 'height' must be numbers!")

	return p
end

function _M:draw()
	if not self.visible then return end
	
	local lg = love.graphics
	local absx, absy = self:abspos()

	local sis = {lg.getScissor()}
	lg.setScissor(absx, absy, self.width, self.height)
	lg.translate(self.x, self.y)

	self:ondraw()

	lg.setScissor(unpack(sis))
	
	local child
	if self.children then
		for i=1, #self.children do
			child = self.children[i]
			if child.visible then
				child:draw() end
		end
	end
	lg.translate(-self.x, -self.y)
end

function _M:ondraw()
	local lg = love.graphics
	lg.setColorMode("modulate")

	if self.background then
		lg.setColor(self.color_bg)
		local ih, iw = self.background:getHeight(), self.background:getWidth()
		local sh, sw = self.height, self.width
		lg.draw(self.background, 0, 0, 0, sh/ih, sw/iw)
	else
		lg.setColor(self.color_bg)
		lg.rectangle('fill', 0, 0, self.width, self.height)
	end

	lg.setColor(self.color_fg)

	if self.border then
		util.drawborder(self)
	end
	

	local msg = self.text or self.name or string.format("Panel#%d", self.id)
	local f = util.fonts[self.font]
	lg.setFont(f)
	local w, l = f:getWrap(msg, self.width)
	if self.valign == 'top' then
		lg.printf(msg, 0, 0, self.width, self.align)
	else
		local offset = (l*f:getHeight())
		if self.valign == 'middle' then
			offset = ((-offset) / 2) + (self.height / 2)
			lg.printf(msg, 0, offset, self.width, self.align)
	
		else -- valign == bottom
			offset = (-offset) + self.height
			lg.printf(msg, 0, offset, self.width, self.align)
		end
	end
	
end

function _M:hover(x, y)
	if not self.visible then return false end
	if not bbox(self.x, self.y, self.width, self.height, x, y) then
		if self.hovering then
			self.hovering = false
			self:onmouseexit(x-self.x, y-self.y)
		else
			self.hovering = false
		end
		return false
	else
		if not self.hovering then
			self.hovering = true
			self:onmouseenter(x-self.x, y-self.y)
		else
			self.hovering = true
		end
		self:onmouseenter(x-self.x, y-self.y)
	end
--	print("hover", self.name, self.visible, x, y)
	
	
	local child
	for i=#self.children, 1, -1 do
		child = self.children[i]:hover(x-self.x,y-self.y)
		if child then return child end
	end

	-- if none of the children matched, then we're it; return self.
	return self
end

function _M:onmouseenter(x, y) end
function _M:onmouseexit(x, y) end

function _M:uiRoot()
	if self.parent then
		return self.parent:uiRoot()
	else
		return self -- if we have no parent, then we're as root as it gets. as far as we know.
	end
end

function _M:focused()
--	if (self.isFocus and self.visible) then return false end
--	print("checking "..self.name)

	if self.hasFocus then
		local f = self.hasFocus:focused()
--		print(f.name.." has focus")
		return f
	end
	print(self.name.. " has final focus")
	return self	
end

function _M:focus()
	assert(self.visible, "Cannot give focus to invisible elements.")
	self.hasFocus = nil
	if self.parent then
		self.parent:focus()
		self.parent.hasFocus = self
	end
	
	self:onfocus()
	return true
end

function _M:blur()
	print("blurring "..self.name..".")
	if self.parent then 
		self.parent.hasFocus = nil
		self:onblur()
	else
		return false
	end
end

function _M:onfocus() end
function _M:onblur() end


function _M:update(dt)
	if not self.visible then return false end
	self:onupdate(dt)
	
	for i=#self.children, 1, -1 do
		self.children[i]:update(dt)
	end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onupdate() end 


function _M:click(x, y, btn)
	if not self.visible then return false end
	local h = self:hover(x, y)
	if h then return h:onclick(x-self.x, y-self.y, btn) end
	return false
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onclick(...) end 

function _M:mousepressed(x, y, btn)
	if not self.visible then return false end
	local h = self:hover(x, y)
	if h then
		h:focus()
		return h:onmousepressed(x-self.x, y-self.y, btn)
	end
	return false
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onmousepressed(...)end 

function _M:mousereleased(x, y, btn)
	if not self.visible then return false end
	local h = self:hover(x, y)
	if h then return h:onmousereleased(x-self.x, y-self.y, btn) end
	return false
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onmousereleased(...) self:onclick(...)end 

function _M:keypressed(k, u)
	if not self.visible then return false end
	local f = self:focused()
	if f then
		return f:onkeypressed(k, u)
	else
		print('keypressed', "nothing had focus")
	end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onkeypressed() end 

function _M:keyreleased(k)
	if not self.visible then return false end
	local f = self:focused()
	if f then
		return f:onkeyreleased(k)
	else
		print('keypressed', "nothing had focus")
	end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onkeyreleased() end 

function _M:joystickpressed(j, btn)
	if not self.visible then return false end
	local f = self:focused()
	if f then return f:onjoystickpressed(k, u) end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onjoystickpressed(j, btn) end 

function _M:keyjoystickreleased(j, btn)
	if not self.visible then return false end
	local f = self:focused()
	if f then return f:onjoystickreleased(k, u) end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onjoystickreleased(j, btn) end 

function _M:resize(width, height)
	if not self.visible then return false end
	
	if not self:onresize(width, height) then
		self.width, self.height = width, height
	end
	
	local child
	for i=#self.children, 1, -1 do
		child = self.children[i]
		if child.autosize then
			child:resize(child.width, child.height)
		end
	end
end

-- dummy so nothing crashes if instances don't supply their own.
function _M:onresize(x, y) end 

function _M:add(p)
	assert(p and type(p) == 'table', "Can only add valid Panels")
	assert(p ~= self, "A panel cannot be its own child")
	print(string.format("adding %s to %s", p.name, self.name))
	table.insert(self.children, p)
	p.parent = self
end

function _M:addModal(p)
	local np = _M:new{x=0, y=0, width=self.width, height=self.height, border=false, color_bg=util.color(0,0,0,.3)}
	np:add(p)
	function np:onupdate()
		if #self.children < 1 then self:close() end
	end
	self:add(np)
end

function _M:remove(p)
	for k,v in pairs(self.children) do
		if v == p then
			print(string.format("removing %s from %s", p.name, self.name))
			table.remove(self.children, k)
			p.parent = nil
			break
		end
	end
end

function _M:abspos()
	local x, y
	if self.parent then
		x, y = self.parent:abspos()
		return x+self.x, y+self.y
	else
		return self.x, self.y
	end
end

function _M:infobox(x, y)
--	local x, y = self:abspos()
	if type(self.info) == 'table' then
		util.infobox(self.info, x, y, self.scale);
	elseif type(self.info) == 'string' then
		local ti = {name=self.text or self.name;text = self.info}
		util.infobox(ti, x, y, self.scale);
	end
end

function _M:close()
	if (not self:onclose() ) and self.parent then
		self.parent:remove(self)
	end
end

function _M:onclose() end

local function update(...)	ui:update(...) end

local function draw(...)
	local sh, sw = love.graphics.getHeight(), love.graphics.getWidth()
	love.graphics.reset()
	ui:draw()
	
	local x, y = love.mouse.getPosition()

	local t = ui:hover(x, y)
	if t then t:infobox(x, y) end
end

local function keypressed(...) ui:keypressed(...) end
local function keyreleased(...) ui:keyreleased(...) end
local function mousepressed(...) ui:mousepressed(...) end
local function mousereleased(...) ui:mousereleased(...) end
local function joystickpressed(...) ui:joystickpressed(...) end
local function joystickreleased(...) ui:joystickreleased(...) end

function _M:ensconce(t)
	t = t or love
	
	t.update           = update
	t.draw             = draw
	t.keypressed       = keypressed
	t.keyreleased      = keyreleased
	t.mousepressed     = mousepressed
	t.mousereleased    = mousereleased
	t.joystickpressed  = joystickpressed
	t.joystickreleased = joystickreleased
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

