-------------------------------------------------------------------------
-- [util.lua]
-- panel.util
-------------------------------------------------------------------------
-- Copyright (C) Textmode DMB 2011
-------------------------------------------------------------------------
local _M = {_NAME = "util", _TYPE = 'module'}
-------------------------------------------------------------------------

_M.fonts = {}
local fonts_mt = {__index = function(s, k)
	if type(k) ~= 'number' then return nil end
	local f = love.graphics.newFont(k)
	rawset(s, k, f)
	return f
end
}setmetatable(_M.fonts, fonts_mt)

local fonts = _M.fonts

function _M.round(n)
	if n < 0 then return math.floor(n)
	else return math.ceil(n)
	end
end

function _M.color(r, g, b, a)
	local round = _M.round
	assert(r and g and b, "Must give RGB(A) values.")
	return {round(r*255),round(g*255),round(b*255),round((a or 1)*255),}
end

function _M.grey(r, g, b, a)
	if type(r) == 'table' then r, g, b = unpack(r) end
	
	local v = math.max(r, g, b)
--	local l = (r+g+b)/(255*3)
	return v, v, v, a
end

function _M.RGBA2HSVA(r, g, b, a)
	r, g, b, a = r/255, g/255, b/255, (a or 255) / 255
	local h, s, v
	local min, max = math.min(r, g, b), math.max(r, g, b)


	v, s = max, (max - min)
	if v == 0 then return 0, 0, 0, a end
	if s == 0 then return 0, 0, v, a end

	local r, g, b = r/v, g/v, b/v
	if (max == r) then
		h = 0.0 + 60 * (g-b)
		if (h < 0.0) then
			h = h+360 end
	elseif (max == g) then
		h = 120 + 60.0*(b-r)
	else -- max == b
		h = 240 + 60*(r-g)
	end
	  
	return _M.round(h), _M.round(s*100), _M.round(v*100), _M.round(a*100)
end

function _M.HSVA2RGBA(h, s, v, a)
	h = h / 360
	s = s / 100
	v = v / 100
	a = (a or 100) / 100
	  
	if s == 0 then  -- zero-sat == grey
		v = v _M.round(v*255)
		return v, v, v, a
	end

	h=h*6
	local i = math.floor(h)
	f = h - i 
	p = v * ( 1 - s )
	q = v * ( 1 - s * f )
	t = v * ( 1 - s * (1 - f) )

	local r, g, b

	if     i == 0 then r, g, b = v, t, p
	elseif i == 1 then r, g, b = q, v, p
	elseif i == 2 then r, g, b = p, v, t
	elseif i == 3 then r, g, b = p, q, v
	elseif i == 4 then r, g, b = t, p, v
	else               r, g, b = v, p, q
	end
	  
	return _M.round(r*255), _M.round(g*255), _M.round(b*255), _M.round(a*255)
end

function _M.bbox(bx, by, bw, bh, tx, ty)
	return (tx >= bx and tx <= (bx+bw)) and (ty >= by and ty <= (by+bh))
end

local font = 10
local bg_col = _M.color(.51,.75,.94, .66);


function _M.infobox(info, x, y, s)
	assert(info and (x and y), "Must be given a drawable card, and coords")
	assert(type(info) == 'table', "infobox: parm#1 expected table, got "..type(info))
	assert(type(x) == 'number', "infobox: parm#2 expected number, got "..type(x))
	assert(type(y) == 'number', "infobox: parm#3 expected number, got "..type(y))
	assert((not s) or type(s) == 'number', "infobox: parm#4 expected number, got "..type(s))
	if info.active == false or not (info.active and info.name)  then return end
	assert(info.name and info.text, "Info must have a name, and text, or be inactive")
	s = s or 1
	local w = info.width or 320
--	print("getWrap:", fonts[font*s]:getWrap(info.text, w))
	local f = fonts[font*s]
	local w, l = f:getWrap(info.text, w)
	l = l+1
	w = w*1.2
	local h = info.height or ((l+1.5) * f:getHeight()) + ((fonts[font*s]:getHeight()+3)*1.2)
	
	local lg = love.graphics

	do
		local sw, sh = lg.getWidth(), lg.getHeight()
		if x+w >= sw then x = sw-w end
		if y+h >= sh then y = sh-h end
	end
	
--	local sis = {lg.getScissor()}
--	lg.setScissor(x, y, w*s, h*s)
	local col = info.color or bg_col
	lg.setColor(col)
	lg.rectangle('fill', x, y, w*s, h*s)
	
	lg.setColorMode('modulate')
	if _M.grey(col) < .5 then
		lg.setColor(255, 255, 255)
	else
		lg.setColor(0, 0, 0)
	end
	lg.rectangle('line', x+(1*s), y+(1*s), (w-1.5)*s, (h-1.5)*s)
	
	local f = lg.getFont()
	lg.setFont(fonts[(font+4)*s])
	lg.print(info.name, x+(3*s), y+(3*s))
	local off = lg.getFont():getHeight()*1.2
	lg.setFont(fonts[font*s])
	lg.printf(info.text, x+(6*s), y+off+(3*s), (w-12)*s)
--	lg.setScissor(unpack(sis))
	lg.setColorMode('replace')

end

function _M.drawborder(t)
	local lg = love.graphics
	local c = {lg.getColor()}
	
	lg.setColor(0,0,0, 100)
	lg.line(.5, .5+t.height-1, .5+t.width-1, .5+t.height-1)
	lg.line(.5+t.width-1, .5, .5+t.width-1, .5+t.height)

	lg.setColor(255,255,255, 100)
	lg.line(.5, .5, .5, t.height)
	lg.line(.5, .5, t.width, .5)
	
	lg.setColor(unpack(c))
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

